@@ -65,10 +65,57 @@ def set_warnings(numpy_level='ignore', astropy_level='ignore'):
     
     np.seterr(all=numpy_level)
     warnings.simplefilter(astropy_level, category=AstropyWarning)
-    
-def running_median(xi, yi, NBIN=10, use_median=True, use_nmad=True, reverse=False, bins=None, x_func=astropy.stats.biweight_location, y_func=astropy.stats.biweight_location, std_func=astropy.stats.biweight_midvariance, integrate=False):
+
+
+def running_median(xi, yi, NBIN=10, reverse=False, bins=None, x_func=np.median, y_func=np.median, std_func=astropy.stats.mad_std, x_kwargs={}, y_kwargs={}, std_kwargs={}, use_biweight=False, integrate=False, **kwargs):
     """
-    Running median/biweight/nmad
+    Binned median/biweight/nmad statistics
+    
+    Parameters
+    ----------
+    xi : array-like
+        Data of independent variable
+    
+    yi : array-like
+        Data of dependent variable
+    
+    NBIN : int
+        Number of bins along `xi`
+    
+    reverse : bool
+        Calculate bins starting at largest values of `xi`
+    
+    bins : array-like
+        Fixed bins, rather than calculating with `NBIN`
+    
+    x_func : function
+        Function to compute moments of `xi`
+    
+    y_func, std_func : function
+        Functions to compute moments of `yi`.  Assumed to be the central 
+        value and dispersion, but don't have to be
+    
+    x_kwargs, y_kwargs, std_kwargs : dict
+        Keyword arguments to pass to moment functions
+    
+    use_biweight : bool
+        Use robust biweight estimators:
+        
+            - `x_func` : `astropy.stats.biweight_location`
+            - `y_func` : `astropy.stats.biweight_location`
+            - `std_func` : `astropy.stats.biweight_midvariance`
+        
+    integrate : bool
+        Numerically integrate `yi` with the trapezoidal rule within the bins
+    
+    Returns
+    -------
+    xm, ym, ys : array-like
+        Binned moments of `xi` and `yi`
+    
+    yn : array-like
+        Number of entries per bin
+        
     """
     
     NPER = xi.size // NBIN
@@ -84,17 +131,16 @@ def running_median(xi, yi, NBIN=10, use_median=True, use_nmad=True, reverse=Fals
     
     NBIN = len(bins)-1
            
-    xm = np.arange(NBIN)*1.
-    xs = xm*0
-    ym = xm*0
-    ys = xm*0
-    N = np.arange(NBIN)
+    xm = np.ones(NBIN)
+    xs = np.zeros_like(xm)
+    ym = np.zeros_like(xm)
+    ys = np.zeros_like(xm)
+    N = np.zeros(NBIN, dtype=int)
     
-    if use_median:
-        y_func = np.median
-    
-    if use_nmad:
-        std_func = astropy.stats.mad_std
+    if use_biweight:
+        x_func = astropy.stats.biweight_location
+        y_func = astropy.stats.biweight_location
+        std_func = astropy.stats.biweight_midvariance
         
     #idx = np.arange(NPER, dtype=int)
     for i in range(NBIN):
@@ -109,35 +155,23 @@ def running_median(xi, yi, NBIN=10, use_median=True, use_nmad=True, reverse=Fals
             dx = (ma-mi)
             ym[i] = np.trapz(yi[in_bin][xso], xi[in_bin][xso])/dx
         else:
-            xm[i] = x_func(xi[in_bin])
-            ym[i] = y_func(yi[in_bin])
+            xm[i] = x_func(xi[in_bin], **x_kwargs)
+            ym[i] = y_func(yi[in_bin], **y_kwargs)
         
-        ys[i] = std_func(yi[in_bin])
-        
-        # if use_median:
-        #     xm[i] = np.median(xi[in_bin]) # [so][idx+NPER*i])
-        #     ym[i] =  np.median(yi[in_bin]) # [so][idx+NPER*i])
-        # else:
-        #     xm[i] = astropy.stats.biweight_location(xi[in_bin]) # [so][idx+NPER*i])
-        #     ym[i] = astropy.stats.biweight_location(yi[in_bin]) # [so][idx+NPER*i])
-        #     
-        # if use_nmad:
-        #     mad = astropy.stats.median_absolute_deviation
-        #     ys[i] = 1.4826*mad(yi[in_bin]) # [so][idx+NPER*i])
-        # else:
-        #     ys[i] = astropy.stats.biweight_midvariance(yi[in_bin]) # [so][idx+NPER*i])
-            
+        ys[i] = std_func(yi[in_bin], **std_kwargs)
+                    
     return xm, ym, ys, N
 
-def nmad(arr):
+
+def nmad(data):
     """
     Normalized median absolute deviation statistic
     
-    Includes the normalization factor of 1.48, see 
-    `astropy.stats.median_absolute_deviation`.
+    Just a wrapper around `astropy.stats.mad_std`.
     """
     import astropy.stats
-    return 1.48*astropy.stats.median_absolute_deviation(arr)
+    #return 1.48*astropy.stats.median_absolute_deviation(arr)
+    return astropy.stats.mad_std(data)
 
 
 def log_zgrid(zr=[0.7,3.4], dz=0.01):
@@ -478,6 +512,56 @@ def abs_mag_to_luminosity(absmag, pivot=None, output_unit=u.L_sun):
 def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[6,7], minor=0.5, skip=2, selection=None, catastrophic_limit=0.15, title=None, min_zphot=0.02, alpha=0.2, extra_xlabel='', extra_ylabel='', xlabel=r'$z_\mathrm{spec}$', ylabel=r'$z_\mathrm{phot}$', label_pos=(0.05, 0.95), label_kwargs=dict(ha='left', va='top', fontsize=10), label_prefix='', format_axes=True, color='k', point_label=None, **kwargs):
     """
     Make zphot_zspec plot scaled by log(1+z) and show uncertainties
+
+    Parameters
+    ----------
+    zphot : array-like
+        Redshift on dependent axis
+
+    zspec : array-like
+        Redshift on independent axis
+
+    zlimits : (N, 2) array
+        Redshifts to use for photo-z errorbars, e.g. from 
+        `~eazy.photoz.Photoz.pz_percentiles`, where `N` is the number of 
+        objects as in `zphot` and `zspec`
+
+    zmin, zmax : float
+        Plot limits
+
+    axes : `matplotlib` axes, None
+        If specified, overplot in existing axes rather than generating a new
+        plot.  For example, run the function once to generate the figure and
+        then plot different points onto the existing axes:
+        
+        >>> fig = eazy.utils.zphot_spec(zphot, zspec, selection=sample1)
+        >>> _ = eazy.utils.zphot_spec(zphot, zspec, selection=sample2, 
+        >>>                           axes=fig.axes, color='b')
+
+    figsize : list
+        Figure canvas dimensions
+
+    minor : float
+        Axis tick interval
+
+    skip : int
+        Put axis labels every `skip` ticks
+
+    selection : array-like
+        Subsample selection (boolean or indices) applied as `zphot[selection]`
+
+    catastrophic_limit : float
+        Limit to define "catastrophic" failures, which is used for computing
+        precision / outlier statistics printed on the plot
+
+    title : str
+        Title to add to the plot axes
+
+    Returns
+    -------
+    fig : `matplotlib.figure.Figure`
+        Figure object
+
     """
     import matplotlib.pyplot as plt
     from matplotlib.gridspec import GridSpec
@@ -604,71 +688,152 @@ def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[
         return True
 
 
-def query_html(ra, dec, **kwargs):
+def query_html(ra, dec, with_coords=True, replace_comma=True, queries=['CDS','ESO','MAST','ALMA', 'LEG','HSC'], **kwargs):
     """
-    Return HTML string of queries at a position
+    Return HTML string of queries around a position
+    
+    Parameters
+    ----------
+    ra, dec : float
+        Coordinates in decimal degrees
+    
+    with_coords : bool
+        Include '(ra, dec)' in output string
+    
+    replace_comma : bool
+        Replace ',' with URL-safe '%2C'
+    
+    queries : list
+        - CDS: Vizier/CDS catalogs
+        - ESO: ESO archive
+        - MAST: STScI/MAST HST archive
+        - ALMA: ALMA archive
+        - LEG/LEGACY: LegacySurvey map interface
+        - HSC: HSC map interface
+    
+    Returns
+    -------
+    html : str
+        HTML-formatted string with query links
+        
     """
-    html = f"({ra:.6f}, {dec:.6f}) "
-    for func, name in zip([cds_query, eso_query, mast_query, alma_query, show_legacysurvey, hscmap_query], ['CDS','ESO','MAST','ALMA', 'LEG','HSC']):
+    if with_coords:
+        html = [f"({ra:.6f}, {dec:.6f})"]
+    else:
+        html = []
+    
+    # Function/name mapping
+    funcs = [cds_query, eso_query, mast_query, alma_query, show_legacysurvey, hscmap_query]
+    names = ['CDS','ESO','MAST','ALMA', 'LEG','HSC']
+    query_map = {}
+    for name, func in zip(names, funcs):
+        query_map[name] = func
+    
+    query_map['LEGACY'] = query_map['LEG']
+    
+    for name in queries:
+        if name in query_map:
+            func = query_map[name]
+        else:
+            continue
+            
         url = func(ra, dec, **kwargs)
-        html += f' <a href="{url}">{name}</a>'
+        html.append(f'<a href="{url}">{name}</a>')
     
+    html = ' '.join(html)
+    
+    if replace_comma:
+        html = html.replace(',','%2C')
+        
     return html
-    
-def cds_query(ra, dec, radius=1., **kwargs):
+
+
+def cds_query(ra, dec, radius=1., unit='s', **kwargs):
     """
     Open browswer with CDS catalog query around central position
+    
     """
     #rd = self.get('pan fk5').strip()
     rd = f'{ra} {dec}'
     rdst = rd.replace('+', '%2B').replace('-', '%2D').replace(' ', '+')
     url = (f'http://vizier.u-strasbg.fr/viz-bin/VizieR?'
-           f'-c={rdst}&-c.rs={radius:.1f}')
+           f'-c={rdst}&-c.r{unit}={radius}')
            
     #os.system(f'open {url}')
     return url
 
-def eso_query(ra, dec, radius=1., dp_types=['CUBE','IMAGE'], extra='', **kwargs):
+
+def eso_query(ra, dec, radius=1., unit='m', dp_types=['CUBE','IMAGE'], extra='', **kwargs):
     """
     Open browser with ESO archive query around central position.
     
-    ``radius`` in arcmin.
+    Note: ESO query is data footprint **contains*** point
+    
     """
     #ra, dec = self.get('pan fk5').strip().split()
     
+    # native is deg
+    if unit == 'd':
+        r = f'{radius:.2f}'
+    elif unit == 'm':
+        r = f'{radius/60:.3f}'
+    elif unit == 's':
+        r = f'{radius/3600:.5f}'
+        
     dp_type = ','.join(dp_types)
     
     url = (f'https://archive.eso.org/scienceportal/home?'
-            f'pos={ra},{dec}&r={radius/60.}&dp_type={dp_type}{extra}')
+            f'pos={ra},{dec}&r={r}&dp_type={dp_type}{extra}')
                     
     #os.system(f'open {url}')
     return url
 
-def mast_query(ra, dec, instruments=['WFC3','ACS','WFPC2'], max=1000, **kwargs):
+
+def mast_query(ra, dec, instruments=['WFC3','ACS','WFPC2'], mast_radius=1., mast_unit='m', max=1000, **kwargs):
     """
     Open browser with MAST archive query around central position
+    
+    Note: MAST query is **distance to** point
+    
     """
     #ra, dec = self.get('pan fk5').strip().split()
     if len(instruments) > 0:
         instr='&sci_instrume='+','.join(instruments)
     else:
         instr = ''
+    
+    # native is arcmin
+    if mast_unit == 'd':
+        r = f'{mast_radius*60:.2f}'
+    elif mast_unit == 'm':
+        r = f'{mast_radius:.3f}'
+    elif mast_unit == 's':
+        r = f'{mast_radius/60:.5f}'
         
     url = (f'https://archive.stsci.edu/hst/search.php?RA={ra}&DEC={dec}'
+           f'&radius={r}'
            f'&sci_aec=S{instr}&max_records={max}&outputformat=HTML_Table'
             '&action=Search')
             
     #os.system(f'open {url}')
     return url
 
-def alma_query(ra, dec, mirror="almascience.eso.org", radius=1, extra='', **kwargs):
+
+def alma_query(ra, dec, mirror="almascience.eso.org", radius=1, unit='m', extra='', **kwargs):
     """
     Open browser with ALMA archive query around central position
     """
-    #ra, dec = self.get('pan fk5').strip().split()
 
-    url = (f"https://{mirror}/asax/?result_view=observation"
-           f"&raDec={ra}%20{dec},{radius}{extra}")
+    # native is arcmin
+    if unit == 'd':
+        r = f'{radius*60:.2f}'
+    elif unit == 'm':
+        r = f'{radius:.3f}'
+    elif unit == 's':
+        r = f'{radius/60:.5f}'
+        
+    url = (f"https://{mirror}/aq/?result_view=observation"
+           f"&raDec={ra}%20{dec},{r}{extra}")
     #os.system(f'open "{url}"')
     return url
 
@@ -702,7 +867,8 @@ def show_legacysurvey(ra, dec, layer='dr8', zoom=17, **kwargs):
             
     #os.system(f'open {url}')
     return url
-    
+
+
 def interp_conserve(x, xp, fp, left=0., right=0.):
     """
     Interpolation analogous to `numpy.interp` but conserving "flux".
@@ -753,7 +919,8 @@ def interp_conserve(x, xp, fp, left=0., right=0.):
         outy[i] = np.trapz(fully[bin], fullx[bin])/dx[i]
         
     return outy
-        
+
+
 class emceeChain():     
     def __init__(self, chain=None, file=None, param_names=[],
                        burn_fraction=0.5, sampler=None):
